using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.Serialization;
using Syncfusion.Windows.Forms.Grid;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections;

namespace VirtualGridIssue
{

    [Serializable]
    public class EnumCellModel : GridComboBoxCellModel
    {
        protected EnumCellModel(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }

        public EnumCellModel(GridModel grid)
            : base(grid)
        {
            AllowDoubleClickChangeSelectedIndex = false;
        }

        public override GridCellRendererBase CreateRenderer(GridControlBase control)
        {
            return new EnumCellRenderer(control, this);
        }

        //used to provide optimal height and width for cell (as in grid.ColWidths.ResizeToFit calls)
        protected override Size OnQueryPrefferedClientSize(Graphics g, int rowIndex, int colIndex, GridStyleInfo style, GridQueryBounds queryBounds)
        {
            Size sz = base.OnQueryPrefferedClientSize(g, rowIndex, colIndex, style, queryBounds);

            //adjust to allow for button width
            sz.Width = sz.Width + ButtonBarSize.Width;

            return sz;
        }
    }

    public class EnumCellRenderer : GridComboBoxCellRenderer
    {
        #region Data Members

        private object m_OldValue;
        GridCellButton m_TempButton = null;
        GridCellModelBase m_Model;
        Size m_ButtonBarSize = Size.Empty;


        #endregion // Data Members

        public EnumCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
            : base(grid, cellModel)
        {
            m_TempButton = DropDownButton;
            m_Model = cellModel;
            m_ButtonBarSize = m_Model.ButtonBarSize;
        }

        protected override bool OnSaveChanges()
        {
            System.Type currentType = Grid.Model[RowIndex, ColIndex].CellValue.GetType();
            TypeConverter conv = TypeDescriptor.GetConverter(currentType);
            if (conv != null)
            {
                Grid.Model[RowIndex, ColIndex].CellValue = conv.ConvertFromString(TextBox.Text);
                return true;
            }
            return base.OnSaveChanges();
        }

        protected override void OnInitialize(int rowIndex, int colIndex)
        {
            base.OnInitialize(rowIndex, colIndex);
            if (Grid.Model[rowIndex, colIndex].CellValue != null)
            {
                if (ControlValue.GetType().IsEnum)
                {
                    ApplyListBoxStyle();
                }
                else
                {
                    ApplyTextBoxStyle();
                }
            }
            else
                ApplyTextBoxStyle();
        }

        private void ApplyListBoxStyle()
        {
            m_Model.ButtonBarSize = m_ButtonBarSize;
            RemoveButton(m_TempButton);
            if (m_TempButton != null)
                AddButton(m_TempButton);
            TypeConverter conv = TypeDescriptor.GetConverter(ControlValue);
            if (conv != null)
            {
                ListBoxPart.DataSource = new ArrayList(conv.GetStandardValues());
                this.SupportsFocusControl = true;
                this.StyleInfo.DropDownStyle = GridDropDownStyle.Exclusive;
            }
        }

        private void ApplyTextBoxStyle()
        {
            m_TempButton = DropDownButton;
            if (m_TempButton != null)
                RemoveButton(m_TempButton);
            this.TextBox.Size = new Size(10, 10);
            m_Model.ButtonBarSize = new Size(0, 0);

            this.SupportsFocusControl = true;
            this.StyleInfo.DropDownStyle = GridDropDownStyle.Editable;
        }

        public override bool ProcessMouseWheel(MouseEventArgs e)
        {
            if (this.IsDroppedDown)
            {
                if (e.Delta < 0)
                    this.ListBoxPart.TopIndex++;
                else
                    this.ListBoxPart.TopIndex--;
                return true;
            }
            return false;
        }
    }

}
