using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;

namespace VirtualGridIssue
{
    public partial class Form1 : Form
    {
        private List<MyData> m_List;
        private int m_ColCount = 2;

        public Form1()
        {
            InitializeComponent();
            m_List = new List<MyData>();
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            gridControl1[0, 1].Text = "Column 1";
            gridControl1[0, 2].Text = "Column 2";

            gridControl1.ColWidths[0] = 0;

            for (int i = 0; i < 10; i++)
            {
                MyData data = new MyData();
                data.FirstColumn = "First Column Name";
                if (i % 2 == 0)
                    data.SecondColumn = MyEnum.ThisIsMyFirstEnumValue;
                else
                    data.SecondColumn = MyEnum.ThisIsMySecondEnumValue;

                m_List.Add(data);
            }


            // Using the grid in virtual mode
            gridControl1.ResetVolatileData();

            gridControl1.CellModels.Add("EnumCell", new EnumCellModel(gridControl1.Model));

            gridControl1.QueryRowCount += new GridRowColCountEventHandler(OnGridQueryRowCount);
            gridControl1.QueryColCount += new GridRowColCountEventHandler(OnGridQueryColCount);
            gridControl1.QueryCellInfo += new GridQueryCellInfoEventHandler(OnGridQueryCellInfo);
            gridControl1.QueryColWidth += new GridRowColSizeEventHandler(GridQueryColWidth);

            GridRangeInfo info = GridRangeInfo.Cells(0, 1, gridControl1.RowCount, gridControl1.ColCount);

            gridControl1.Model.ColWidths.ResizeToFit(info, GridResizeToFitOptions.IncludeHeaders |
                GridResizeToFitOptions.IncludeCellsWithinCoveredRange |
                GridResizeToFitOptions.ResizeCoveredCells);
        }

        protected virtual void GridQueryColWidth(object sender, GridRowColSizeEventArgs e)
        {
            if (e.Index == gridControl1.ColCount)
            {
                int width = gridControl1.ColCount <= 0 ? 0 : gridControl1.ColWidths.GetTotal(1, gridControl1.ColCount - 1);
                e.Size = gridControl1.ClientRectangle.Width - width;
                e.Handled = true;
            }
        }

        void OnGridQueryColCount(object sender, GridRowColCountEventArgs e)
        {
            e.Count = m_ColCount;
            e.Handled = true;
        }

        void OnGridQueryRowCount(object sender, GridRowColCountEventArgs e)
        {
            e.Count = m_List.Count;
            e.Handled = true;
        }

        void OnGridQueryCellInfo(object sender, GridQueryCellInfoEventArgs e)
        {
            if (e.RowIndex > 0 && e.ColIndex > 0)
            {
                if (e.ColIndex == 1)
                {
                    e.Style.CellValue = m_List[e.RowIndex - 1].FirstColumn;
                }
                else if (e.ColIndex == 2)
                {
                    if (m_List[e.RowIndex - 1].SecondColumn.GetType().IsEnum)
                        e.Style.CellType = "EnumCell";
                    e.Style.CellValue = m_List[e.RowIndex - 1].SecondColumn;
                }

            }
        }
    }

    public class MyData
    {
        private string m_FirstColumn;

        public string FirstColumn
        {
            get { return m_FirstColumn; }
            set { m_FirstColumn = value; }
        }

        private MyEnum m_SecondColumn;

        public MyEnum SecondColumn
        {
            get { return m_SecondColumn; }
            set { m_SecondColumn = value; }
        }

    }

    public enum MyEnum
    {
        ThisIsMyFirstEnumValue,
        ThisIsMySecondEnumValue
    }
}